#!/bin/bash
BUILD_DIR=build

function exitWithMessage
{
	MESSAGE=$1
	echo $MESSAGE
	exit 1
}

function checkArchives
{
	echo "------Checking archives"
	pushd resources
	YAML_ARCHIVE=archives/yaml-cpp-0.5.1.tar.gz
	OPENXCOM_ARCHIVE=archives/openxcom.tar.gz
	if [ ! -e $OPENXCOM_ARCHIVE ] || [ ! -e $YAML_ARCHIVE ]
	then
		exitWithMessage "Source archives not found, please fetch first"
	fi
	CHECKSUM_FILE=archives.sha256

	sha256sum -c $CHECKSUM_FILE || exitWithMessage "Archive checksums are incorrect!"
	popd
}

function setUpBuildDir
{
	if [ -e $BUILD_DIR ]
	then
		exitWithMessage "Existing build folder...exiting"
	fi
		
	mkdir $BUILD_DIR
	cd $BUILD_DIR
}

function extractArchives
{
	echo "------Extracting from the source archives"
	
	YAML_ARCHIVE=../resources/archives/yaml-cpp-0.5.1.tar.gz
	OPENXCOM_ARCHIVE=../resources/archives/openxcom.tar.gz
	tar -xzf $YAML_ARCHIVE
	tar -xzf $OPENXCOM_ARCHIVE
}

YAML_FOLDER=yaml-cpp-release-0.5.1

function buildYaml
{
	echo "------Building Yaml-cpp"
	
	pushd $YAML_FOLDER
	mkdir build
	cd build
	cmake .. || exit 1
	make -j$(nproc) yaml-cpp || exit 1
	
	popd
}

OPENXCOM_DIR=openxcom-1.0.0
OPENXCOM_SRC_DIR=$OPENXCOM_DIR/src

function AddYAMLComponentsToOpenXCom
{
	YAML_STATIC=$YAML_FOLDER/build/libyaml-cpp.a
	YAML_INCLUDE_DIR=$YAML_FOLDER/include
	YAML_PC=../resources/yaml-cpp.pc
	OPENXCOM_YAML_DIR=$OPENXCOM_SRC_DIR/yaml
	mkdir $OPENXCOM_YAML_DIR
	OPENXCOM_YAML_LIB_DIR=$OPENXCOM_YAML_DIR/lib
	mkdir $OPENXCOM_YAML_LIB_DIR
	cp $YAML_STATIC $OPENXCOM_YAML_LIB_DIR
	cp -r $YAML_INCLUDE_DIR $OPENXCOM_YAML_DIR
	cp $YAML_PC $OPENXCOM_YAML_DIR
}

function patchOpenXcom
{
	RESOURCE_OPENXCOM_BATTLEUNIT_CPP=../resources/openxcom/BattleUnit.cpp
	BATTLEUNIT_DIR=$OPENXCOM_SRC_DIR/Savegame/
	cp -f $RESOURCE_OPENXCOM_BATTLEUNIT_CPP $BATTLEUNIT_DIR
}

function setUpOpenXcomBuild
{
	echo "------Setting up openxcom for building"
	
	AddYAMLComponentsToOpenXCom
	RESOURCE_OPENXCOM_MAKEFILE=../resources/openxcom/Makefile_static.simple
	RESOURCE_OPENXCOM_BUILD_SCRIPT=../resources/openxcom/build.sh
	cp $RESOURCE_OPENXCOM_MAKEFILE $OPENXCOM_SRC_DIR
	cp $RESOURCE_OPENXCOM_BUILD_SCRIPT $OPENXCOM_SRC_DIR

	#Optional: patch BattleUnit.cpp to use stat improvement corresponding to http://www.ufopaedia.org/index.php?title=Experience#Primary_Stats 
	#patchOpenXcom
	mkdir $OPENXCOM_DIR/obj
}

function buildOpenXcom
{
	echo "------Building openxcom"
	pushd $OPENXCOM_SRC_DIR
	bash build.sh || exit 1
	popd
}


checkArchives
setUpBuildDir

extractArchives
buildYaml
setUpOpenXcomBuild
buildOpenXcom
cp -r $OPENXCOM_DIR/bin ../bin

echo "------------------"
echo "Build completed, don't forget to copy the game data to bin/data"
echo "See build/$OPENXCOM_DIR/README.txt"
